@@ -37,17 +37,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <pcre.h>
-
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <time.h>
 
 #include <libprelude/prelude-log.h>
 
